@@ -28,6 +28,13 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <fcntl.h>
+
+#ifdef _MSC_VER
+#  include <intrin.h>
+#endif
+
 #include <orc/orcdebug.h>
 #include <orc/orcsse.h>
 #include <orc/orcmmx.h>
@@ -36,13 +43,6 @@
 
 #include "orcinternal.h"
 
-#include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <time.h>
-
-
 int orc_x86_sse_flags;
 int orc_x86_mmx_flags;
 static orc_uint32 orc_x86_vendor;
@@ -207,8 +207,9 @@ static void
 orc_x86_detect_cpuid (void)
 {
   static int inited = 0;
-  orc_uint32 ebx, edx;
-  orc_uint32 level;
+  orc_uint32 ebx = 0;
+  orc_uint32 edx = 0;
+  orc_uint32 level = 0;
 
   if (inited) return;
   inited = 1;
@@ -328,13 +329,18 @@ orc_x86_cpuid_handle_standard_flags (void)
     orc_x86_sse_flags |= ORC_TARGET_SSE_SSE4_2;
   }
 
-  if (ecx & (1 << 28)) {
+  // Linux mitigation for GDS requires checking for XSAVE too.
+  // https://bugzilla.mozilla.org/show_bug.cgi?id=1854795
+  // https://gitlab.freedesktop.org/gstreamer/orc/-/issues/65
+  const int xsave_enabled = ecx & (1 << 26);
+
+  if (ecx & (1 << 28) && xsave_enabled) {
     orc_x86_sse_flags |= ORC_TARGET_AVX_AVX;
   }
 
   get_cpuid (0x00000007, &eax, &ebx, &ecx, &edx);
 
-  if (ebx & (1 << 5)) {
+  if (ebx & (1 << 5) && xsave_enabled) {
     orc_x86_sse_flags |= ORC_TARGET_AVX_AVX2;
   }
 }
@@ -490,7 +496,10 @@ orc_sse_detect_cpuid_intel (orc_uint32 level)
 static void
 orc_sse_detect_cpuid_amd (orc_uint32 level)
 {
-  orc_uint32 eax, ebx, ecx, edx;
+  orc_uint32 eax = 0;
+  orc_uint32 ebx = 0;
+  orc_uint32 ecx = 0;
+  orc_uint32 edx = 0;
 
   if (level >= 1) {
     orc_x86_cpuid_handle_standard_flags ();
