@@ -24,8 +24,7 @@ orc_executor_new (OrcProgram *program)
   ex = malloc(sizeof(OrcExecutor));
   memset(ex,0,sizeof(OrcExecutor));
 
-  ex->program = program;
-  ex->arrays[ORC_VAR_A2] = program->orccode;
+  orc_executor_set_program (ex, program);
 
   return ex;
 }
@@ -39,7 +38,7 @@ orc_executor_free (OrcExecutor *ex)
 void
 orc_executor_run (OrcExecutor *ex)
 {
-  void (*func) (OrcExecutor *);
+  OrcExecutorFunc func = NULL;
 
   if (ex->program) {
     func = ex->program->code_exec;
@@ -58,7 +57,7 @@ orc_executor_run (OrcExecutor *ex)
 void
 orc_executor_run_backup (OrcExecutor *ex)
 {
-  void (*func) (OrcExecutor *);
+  OrcExecutorFunc func = NULL;
 
   if (ex->program) {
     func = ex->program->backup_func;
@@ -83,6 +82,7 @@ orc_executor_set_program (OrcExecutor *ex, OrcProgram *program)
   } else {
     ex->arrays[ORC_VAR_A1] = (void *)orc_executor_emulate;
   }
+  ex->arrays[ORC_VAR_A2] = program->orccode;
 }
 
 void
@@ -124,7 +124,7 @@ void
 orc_executor_set_param_int64 (OrcExecutor *ex, int var, orc_int64 value)
 {
   ex->params[var] = value & 0xffffffff;
-  ex->params[var + (ORC_VAR_T1-ORC_VAR_P1)] = (value >> 32) & 0xffffffff;
+  ex->params[var + (ORC_N_PARAMS)] = (value >> 32) & 0xffffffff;
 }
 
 void
@@ -133,7 +133,7 @@ orc_executor_set_param_double (OrcExecutor *ex, int var, double value)
   orc_union64 u;
   u.f = value;
   ex->params[var] = u.i & 0xffffffff;
-  ex->params[var + (ORC_VAR_T1-ORC_VAR_P1)] = (u.i >> 32) & 0xffffffff;
+  ex->params[var + (ORC_N_PARAMS)] = (u.i >> 32) & 0xffffffff;
 }
 
 void
@@ -297,7 +297,7 @@ orc_executor_emulate (OrcExecutor *ex)
           load_constant (tmpspace[insn->src_args[k]], 8,
               (orc_uint64)(orc_uint32)ex->params[insn->src_args[k]] |
               (((orc_uint64)(orc_uint32)ex->params[insn->src_args[k] +
-               (ORC_VAR_T1 - ORC_VAR_P1)])<<32));
+               (ORC_N_PARAMS)])<<32));
         } else {
           load_constant (tmpspace[insn->src_args[k]], 8,
               ex->params[insn->src_args[k]]);
