@@ -9,10 +9,10 @@ ORC_BEGIN_DECLS
 
 #ifdef ORC_ENABLE_UNSTABLE_API
 
-/* FIXME: not sure why these are exported, no one needs to call these, that's
- * already done as part of orc_init() */
+/* The function prototypes need to be visible to orc.c */
 void orc_mmx_init (void);
 void orc_sse_init (void);
+void orc_avx_init (void);
 void orc_arm_init (void);
 void orc_powerpc_init (void);
 void orc_c_init (void);
@@ -36,6 +36,8 @@ extern const char *_orc_cpu_name;
 
 void orc_compiler_emit_invariants (OrcCompiler *compiler);
 int orc_program_has_float (OrcCompiler *compiler);
+
+char* _orc_getenv (const char *var);
 #endif
 
 ORC_END_DECLS
